/**
 * 项目：互联网医疗
 * 模型分组：服务管理
 * 模型名称：影像报告表
 * @Author: xiongwei
 * @Date: 2023-09-05 09:42:00
 */

package com.xwd.hospital.server.service.impl;

import java.io.Serializable;
import java.util.List;
import java.util.ArrayList;
import java.util.Collection;

import cn.dev33.satoken.stp.StpUtil;
import com.xwd.hospital.server.domain.MedicalReportPicture;
import com.xwd.hospital.server.domain.User;
import com.xwd.hospital.server.domain.UserInfo;
import com.xwd.hospital.server.dto.ReportDto;
import com.xwd.hospital.server.enums.ReportStateEnum;
import com.xwd.hospital.server.service.MedicalReportPictureService;
import com.xwd.hospital.server.service.UserInfoService;
import com.xwd.hospital.server.service.UserService;
import jakarta.annotation.Resource;
import com.baomidou.mybatisplus.core.enums.SqlMethod;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.xwd.hospital.server.service.MedicalReportService;
import com.xwd.hospital.server.repository.MedicalReportMapper;
import com.xwd.hospital.server.repository.base.MedicalReportBaseMapper;
import com.xwd.hospital.server.domain.MedicalReport;
import com.xwd.hospital.server.rest.req.MedicalReportParam;

@Service
public class MedicalReportServiceImpl extends ServiceImpl<MedicalReportMapper, MedicalReport> implements MedicalReportService {

    @Resource
    private MedicalReportPictureService pictureService;
    @Resource
    private UserInfoService userInfoService;
    @Resource
    private UserService userService;

    @Override
    public int updateAllFieldsById(MedicalReport entity) {
        return this.getBaseMapper().updateAllFieldsById(entity);
    }

    /**
     * 批量插入
     *
     * @param entityList ignore
     * @param batchSize  ignore
     * @return ignore
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveBatch(Collection<MedicalReport> entityList, int batchSize) {
        String sqlStatement = SqlHelper.getSqlStatement(MedicalReportBaseMapper.class, SqlMethod.INSERT_ONE);
        return executeBatch(entityList, batchSize, (sqlSession, entity) -> sqlSession.insert(sqlStatement, entity));
    }

    @Override
    public boolean uploadReport(ReportDto dto) {

        MedicalReport medicalReport = new MedicalReport();
        medicalReport.setAge(dto.getAge());
        medicalReport.setPatientName(dto.getPatientName());
        medicalReport.setReportDoctor(dto.getReportDoctor());
        medicalReport.setReportHospital(dto.getReportHospital());
        medicalReport.setPatientIdNo(dto.getPatientIdNo());
        medicalReport.setSex(dto.getSex());
        medicalReport.setReportItem(dto.getReportItem());
        medicalReport.setReportDeclare(dto.getReportDeclare());
        medicalReport.setReportTime(dto.getReportTime());
        this.save(medicalReport);

        if(null != dto.getPictureList()
            && dto.getPictureList().size()>0){
            ArrayList<MedicalReportPicture> pictureList = new ArrayList<>();
            for (String picUrl:dto.getPictureList()) {
                MedicalReportPicture picture = new MedicalReportPicture();
                picture.setMedicalReportId(medicalReport.getId());
                picture.setReportPicture(picUrl);
                pictureList.add(picture);
            }
            pictureService.saveBatch(pictureList);
        }

        return true;
    }

    @Override
    public MedicalReport myDoctorAsk() {
        //当前用户
        Object userId = StpUtil.getExtra("userId");
        UserInfo userInfo = userInfoService.getOne(Wrappers.<UserInfo>query().eq("user_id", Long.valueOf(userId.toString())));
        if(null == userInfo){
            return null;
        }
        User sysUser = userService.getById(Long.valueOf(userId.toString()));

        //根据medical_report中的身份证号查询和报告状态查询，查询报告状态为INTERPRETED下的最新一条医嘱，如果没有数据则查询报告状态为CONFIRMED下的最新一条医嘱
        MedicalReport medicalReport = this.getOne(Wrappers.<MedicalReport>query()
                .eq("patient_id_no", userInfo.getIdNo())
                .and(i -> i.eq("report_state", ReportStateEnum.INTERPRETED).or().eq("report_state", ReportStateEnum.CONFIRMED))
                .orderByDesc("report_state").orderByDesc("createtime").last("limit 1"));
        return medicalReport;
    }
}
